##Processing BV validation cohort data
library(data.table)
library(ggplot2)
library(KEGGREST)

mets = fread("BV_data/bv_validation_mets.txt", header=T, skip = 3)
met_names1 = mets[,Subject_ID][5:dim(mets)[1]]
met_names1 = gsub(" $","", met_names1)
met_names1 = strsplit(met_names1, "/")
met_ids = vector("list", length(met_names1))
kegg_ids = vector("list", length(met_names1))

###Mapping to KEGG IDs (semi-manual curation)

for(j in 1:length(met_ids)){
  met_ids[[j]] = keggFind("compound", query=met_names1[[j]])
  if(!is.null(met_ids[[j]])) kegg_ids[[j]] = names(met_ids[[j]]) else kegg_ids[[j]] = NA
}

kegg_ids2 = gsub("cpd:","",sapply(kegg_ids, function(x){ if(length(x) >0) return(x[1]) else return(NA)}))

kegg_ids2[4] = kegg_ids[[4]][2]
kegg_ids2[5] = kegg_ids[[5]][2]
kegg_ids2[8] = "C00383"
kegg_ids2[9] = "C00122"
kegg_ids2[10] = "C00042"
kegg_ids2[19] = kegg_ids[[19]][2]
kegg_ids2[25] = "C00407"
kegg_ids2[32] = kegg_ids[[32]][2]
kegg_ids2[44] = "C01678"
kegg_ids2[46] = kegg_ids[[46]][2]
kegg_ids2[47] = kegg_ids[[47]][3]
kegg_ids2[49] = kegg_ids[[49]][4]
kegg_ids2[50] = "C08645"
kegg_ids2[51] = "C03264"
kegg_ids2[53] = kegg_ids[[53]][2]
kegg_ids2[55] = "C05582"
kegg_ids2[56] = kegg_ids[[56]][3]
kegg_ids2[58] = "C00864"
kegg_ids2[62] = kegg_ids[[62]][5]
kegg_ids2[63] = kegg_ids[[63]][9]
kegg_ids2[65] = kegg_ids[[65]][8]
kegg_ids2[67] = kegg_ids[[67]][6]
kegg_ids2[70] = kegg_ids[[70]][2]
kegg_ids2[78] = "C00072"
kegg_ids2[80] = "C01104"
kegg_ids2[82] = "C00989"
kegg_ids2[84] = "C14762"
kegg_ids2[87] = kegg_ids[[87]][3]
kegg_ids2[89] = kegg_ids[[89]][2]
kegg_ids2[90] = "C00191"
kegg_ids2[91] = kegg_ids[[91]][2]
kegg_ids2[92] = kegg_ids[[92]][2]
kegg_ids2[96] = "C00253"
kegg_ids2[100] = kegg_ids[[100]][6]
kegg_ids2[101] = "C00117"
kegg_ids2 = gsub("cpd:","", kegg_ids2)

mets = mets[,which(mets[4]=="main"),with=F]
mets[,KEGG:=c(rep(NA, 4),kegg_ids2)]

spec = fread("BV_data/bv_validation_pyro.txt",skip=6, header=T)
spec = spec[rowSums(spec[,4:73,with=F]) >= 10] #only save taxa with at least 10 reads in the whole study


##Formatting of picrust output

sequence_dir = ""
validation_picrust = fread("predicted_metagenomes.txt")
validation_picrust[,KEGG_Description:=NULL]
sample_map = fread(paste0(sequence_dir, "SraRunTable.txt"))
setkey(sample_map, "Run_s")
setnames(validation_picrust, c("KO", sample_map[names(validation_picrust)[2:length(names(validation_picrust))], Sample_Name_s]))

validation_picrust = validation_picrust[rowSums(validation_picrust[,2:dim(validation_picrust)[2],with=F])!=0]

sample_key2 = fread("BV_data/bv_validation_pyro.txt")[1:6]
sample_key2 = data.table(t(sample_key2))
setkey(sample_key2, V6)
setnames(spec, c(names(spec)[1:3], sample_key2[J(names(spec)[4:length(names(spec))]), V3]))
setnames(mets, c(sample_key2[J(names(mets)[1:dim(mets)[2]-1]), V3], "KEGG"))

good_mets = mets[!is.na(KEGG)]

good_mets = good_mets[,c(sort(names(good_mets)[1:dim(good_mets)[2]-1]),"KEGG"),with=F]
good_spec = spec[,c(names(spec)[1:3], sort(names(spec)[4:dim(spec)[2]])),with=F]
good_kos = validation_picrust[,c("KO", sort(names(validation_picrust)[2:dim(validation_picrust)[2]])), with=F]

#dealing with met N/A values
good_mets[good_mets=="N/A"] = NA
good_mets = good_mets[apply(good_mets, 1,function(x){ length(x[is.na(x)])}) <=25]


write.table(good_mets, file = "BV_data/validation_mets_good.txt", row.names=F, quote=F, sep = "\t")
write.table(good_spec, file = "BV_data/validation_spec_good.txt", row.names=F, quote=F, sep = "\t")
write.table(good_kos, file = "BV_data/validation_picrust_good.txt", row.names=F, quote=F, sep = "\t")

#pplacer data from original BV paper
good_spec = fread("BV_data/validation_spec_good.txt")
rowSums(good_spec[!(V3 %in% c("species","genus")),4:dim(good_spec)[2],with=F])
apply(good_spec[!(V3 %in% c("species","genus")),4:dim(good_spec)[2],with=F],1,median)
apply(good_spec[,4:dim(good_spec)[2],with=F],1,median)
#Going to exclude everything above genus or species level
good_spec = good_spec[V3 %in% c("species","genus")]
write.table(good_spec, file = "BV_data/validation_spec_good.txt", row.names=F, quote=F, sep = "\t")



