#!/bin/bash

##Run all taxa-metabolite analyses

HOMEDIR=""

#Locations for each dataset
DATADIR1="$HOMEDIR/BV_data/"
DATADIR2="$HOMEDIR/CD_mice_data/"
DATADIR3="$HOMEDIR/Swedish_data/"
DATADIR4="$HOMEDIR/Ecoli_data/"

RUNDIR="$HOMEDIR/runs_out" #output dir for CMP scores

#0 generate community metabolic network template using the following two pieces of information:
# - the file reaction_mapformula.lst from KEGG (needs to be in same directory), showing reactions annotated in pathways
# - information from KEGG linking reactions to KOs and providing reaction stoichiometry, can be accessed using either 
# the KEGGREST API R package or files from a downloaded version of KEGG (supply either "KEGGREST" or "file" as argument to specify which)
Rscript clean_network_generation_code.R KEGGREST


##1 run PICRUSt on Datasets 2 and 3
normalize_by_copy_number.py -f -i "$DATADIR1/Dataset2_otu_table.txt" -o "$DATADIR1/Dataset2_normalized_otus.biom"
predict_metagenomes.py -f -i "$DATADIR1/Dataset2_normalized_otus.biom" -o "$DATADIR1/Dataset2_metagenome_predictions.txt"
metagenome_contributions.py -i "$DATADIR1/Dataset2_normalized_otus.biom" -o "$DATADIR1/bv_val_picrust_metagenome_contributions.txt"
sed -i -e "s/#OTU ID/KO/g" "$DATADIR1//Dataset2_metagenome_predictions.txt"
#remove last column from PICRUSt output, remove KOs that are all 0
nf=$(($(head -2 "$DATADIR1/Dataset2_metagenome_predictions.txt" | tail -1 | awk '{print NF}')-1))
cut -f 1-$nf "$DATADIR1//Dataset2_metagenome_predictions.txt" > "$DATADIR1/Dataset2_metagenome_temp.txt"
head -2 "$DATADIR1/Dataset2_metagenome_temp.txt" | tail -1 > "$DATADIR1/Dataset2_metagenome_good.txt"
awk '{
for(i=2;i<=NF;i++) t+=$i; if(t !=0) print $0; t=0;
}' "$DATADIR1/Dataset2_metagenome_temp.txt"  >> "$DATADIR1/Dataset2_metagenome_good.txt"


#Dataset 3
normalize_by_copy_number.py -f -i "$DATADIR2/Dataset3_otu_table.txt" -o "$DATADIR2/Dataset3_normalized_otus.biom"
predict_metagenomes.py -f -i "$DATADIR2/Dataset3_normalized_otus.biom" -o "$DATADIR2/Dataset3_metagenome_predictions.txt"
metagenome_contributions.py -i "$DATADIR2/Dataset3_normalized_otus.biom" -o "$DATADIR2/mice_metagenome_contributions.txt"
sed -i -e "s/#OTU ID/KO/g" "$DATADIR1//Dataset3_metagenome_predictions.txt"
#remove last column from PICRUSt output, remove KOs that are all 0
nf=$(($(head -2 "$DATADIR2/Dataset3_metagenome_predictions.txt" | tail -1 | awk '{print NF}')-1))
cut -f 1-$nf "$DATADIR2//Dataset3_metagenome_predictions.txt" > "$DATADIR2/Dataset3_metagenome_temp.txt"
head -2 "$DATADIR2/Dataset3_metagenome_temp.txt" | tail -1 > "$DATADIR2/Dataset3_metagenome_good.txt"
awk '{
for(i=2;i<=NF;i++) t+=$i; if(t !=0) print $0; t=0;
}' "$DATADIR2/Dataset3_metagenome_temp.txt" > "$DATADIR2/Dataset3_metagenome_good.txt" 



##2 run in-house version of PICRUSt on Dataset 1
#fix line endings and headers
perl -pi -e 's/\r/\n/g' "$DATADIR1/Dataset1_qPCR.txt"
echo -e "Sample\tVisit\tgardnerella.vaginalis\tlactobacillus.crispatus\tlactobacillus.jensenii\tlactobacillus.iners\tBVAB1\tBVAB2\tBVAB3\tmegasphaera.type1\tatopobium.vaginae\tLeptoSneath\tEggerthella.sp1\tprevotella.timonensis\tprevotella.buccalis\tprevotella.amnii" > "$DATADIR1/Dataset1_qPCR_good.txt"
tail -n +2 "$DATADIR1/Dataset1_qPCR.txt" >> "$DATADIR1/Dataset1_qPCR_good.txt"

Rscript BV_picrust.R "$DATADIR1/Dataset1_qpcr_good.txt" "$DATADIR1/BV_ref_kos.txt" "$DATADIR1/BV_kos_qpcr_good.txt"

## Format Dataset 1 metabolites
perl -pi -e 's/\r/\n/g' "$DATADIR1/Dataset1_mets.txt"
cut -f 5- "$DATADIR1/Dataset1_mets.txt" > "$DATADIR1/Dataset1_mets_good.txt"


########### 3 run CMP calculations and comparisons

#BV Dataset 1

gene_file5="$DATADIR1/BV_kos_qpcr_good.txt"
met_file5="$DATADIR1/Dataset1_mets_good.txt"

Rscript PRMT.R --genefile="$gene_file5" -m "$met_file5" -w -p "$RUNDIR/bv_q" -n labKegg -f 30 -z 4 -u 10000


#BV validation Dataset 2
gene_file="$DATADIR1/Dataset2_metagenome_good.txt"
met_file="$DATADIR1/Dataset2_mets.txt"

run_musicc.py "$gene_file" -o "$DATADIR1/Dataset2_picrust_musicc.txt" -n -c learn_model -v

Rscript PRMT.R --genefile="$DATADIR1/Dataset2_picrust_musicc.txt" -m "$met_file" -w -p "$RUNDIR/bv_val_picrust" -n labKegg -f 30 -z 4

# #mice ABs Dataset 3
## Format Dataset 3 metabolites
perl -pi -e 's/\r/\n/g' "$DATADIR2/Dataset3_mets.txt"
cut -f 5- "$DATADIR2/Dataset3_mets.txt" > "$DATADIR2/Dataset3_mets_good.txt"

gene_file6="$DATADIR2/Dataset3_metagenome_good.txt"
met_file6="$DATADIR2/Dataset3_mets_good.txt"

run_musicc.py "$gene_file6" -o "$DATADIR2/musicc_out.txt" -n -c learn_model -v

Rscript PRMT.R --genefile="$DATADIR2/musicc_out.txt" -m "$met_file6" -w -p "$RUNDIR/mice_AB" -n labKegg -f 30 -z 4 -u 10000

#Swedish twins Dataset 4
gene_file1="$DATADIR3/genes_good.txt"
met_file1="$DATADIR3/met_data_sub.txt"
met_ids1="$DATADIR3/metabosearch_KEGG_noHMDB.txt"

# #normalize the gene abundance data
run_musicc.py "$gene_file1" -o "$DATADIR3/swedish_musicc_out.txt" -n -c learn_model -v 

Rscript PRMT.R --genefile="$DATADIR3/swedish_musicc_out.txt" -m "$met_file1"  -w --file_prefix="$RUNDIR/swedish" -n labKegg -f 30 -z 4 -i "$met_ids1"

#E coli dataset
gene_file_ecoli="$HOMEDIR/Ecoli_data/genes_sub_tp14_good.txt"
met_file_ecoli="$HOMEDIR/Ecoli_data/mets_sub_tp14_good.txt"

Rscript PRMT.R --genefile="$gene_file_ecoli" -m "$met_file_ecoli"  -w --file_prefix="$RUNDIR/ecoli_tp14" -n labKegg -f 30 -z 4


# 4 Get key species contributors for PICRUSt datasets

#Dataset 2
CONTRIBFILE="$DATADIR1/bv_val_picrust_metagenome_contributions.txt"
RESULTSFILE="$RUNDIR/bv_val_picrust_out.rda"

Rscript singleSpec_contributions_picrust.R "$CONTRIBFILE" "$DATADIR1" "$RESULTSFILE" all make_unnormalized prmts contributions save

#Dataset 3
CONTRIBFILE="$DATADIR2/mice_metagenome_contributions.txt"
RESULTSFILE="$RUNDIR/mice_AB_out.rda"

Rscript singleSpec_contributions_picrust.R "$CONTRIBFILE" "$DATADIR2" "$RESULTSFILE" all make_unnormalized prmts contributions save

###Network shuffling tests (submit to cluster)
qsub -t 1-100 $HOMEDIR/shuffle_net.sh

#Rscript summarize_net_shuff_tests.R bv_q

##Knit Rmd Document with all core analyses and results
Rscript -e "rmarkdown::render('TaxaMet_paper_figures_all.Rmd')"


